##############################################################################
# DBAccess.py - database access module
#    classes:
#
#  (c) 7x Group, LLC.
#            author:  Chris Little, Larry Tuck, and Raymond Billings
#            created: 7/19/2001
#
##############################################################################
import exceptions
import string		# for upper function
import ConfigParser
import os

try:
    import pg		# required for posgreSQL
except StandardError:
    pass

try:
    import DCOracle2	# required for Oracle
except StandardError:
    pass


###################################################################
#  class DBAccessException
#
###################################################################
class DBAccessException(exceptions.Exception):
    """Exception for problems with database access"""

    def __init__(self,args=None):
        self.args = args
#End class DBAccessException


class DBAccess:
    CONFIG_FILE='../config/db.cfg'
    ORACLE='orac'
    POSTGRES='post'

    ######################################################################
    # constructor
    ######################################################################
    def __init__(self,lookupName,db='',configFile=''):
	self.lookupName=lookupName
	self.dbtype=db
	self.config=self.__openConfigFile(configFile)
	self.__connectString=self.__readConfig('connection')
	self.__user=self.__readConfig('user')
	self.__password=self.__readConfig('password')
	self.__port=self.__readConfig('port')
	self.__host=self.__readConfig('host')
        self.__dbname=self.__readConfig('dbname')
	self.debug=self.__readConfig('debug')
	

    ######################################################################
    # getLookupName()
    #
    # return lookupName for this object
    ######################################################################
    def getLookupName(self):
	return self.lookupName


    ######################################################################
    # getDBType()
    #
    # return the database type
    ######################################################################
    def getDBType(self):
	"""return the type of database"""
	return self.dbtype


    ######################################################################
    # setDebug()
    #
    # set debug flag
    ######################################################################
    def setDebug(self,value):
        '''set debug flag'''
        self.debug=value


    ######################################################################
    # __openConfigFile
    #
    # open config file
    ######################################################################
    def __openConfigFile(self, configFile=''):
	config=ConfigParser.ConfigParser()

        if not configFile:
            configFile = DBAccess.CONFIG_FILE
            
	if os.access(configFile,os.R_OK):
	    config.read(configFile)
	else:
	    print "ERROR(DBAccess): Unable to open db.cfg!"
	    config=None

	return config


    ######################################################################
    # __readConfig
    #
    #  read the config file to get the value associated with the
    #  given name, and known application name
    ######################################################################
    def __readConfig(self,option):
	if self.config:
	    try:
		value=self.config.get(self.lookupName,option)
	    except StandardError, e:
		value=self.__getDefault(option)
	        print "ERROR(DBAccess): Unable to read %s,%s from config..." \
                      % (self.lookupName,option)
	    if option=='debug':
		value=int(value)
	else:
	    print "ERROR(DBAccess): No Configuration file!"

	return value
	

    ######################################################################
    #  __oracleExecute
    ######################################################################
    def __oracleExecute(self,query,connection):
	cursor=connection.cursor()

	# Execute Query
	try:
	    cursor.execute(query)
	except StandardError, e:
	    raise DBAccessException, '%s: %s' % (e,query)

	# Get Results
	if string.upper(query[0])=='S':	     # check for select
	    rs=cursor.fetchall()
	    if not rs:
	        rs=[]
	else:
	    rs=[]

	return rs


    ######################################################################
    #  __oracleBinExecute
    ######################################################################
    def __oracleBinExecute(self,query,connection,x):
	cursor=connection.cursor()

	# Execute Query
	try:
	    cursor.execute(query,x=DCOracle2.Binary(x))
	except StandardError, e:
	    raise DBAccessException, '(binExecute)%s: %s' % (e,query)

	# Get Results
	if string.upper(query[0])=='S':	     # check for select
	    rs=cursor.fetchall()
	    if not rs:
	        rs=[]
	else:
	    rs=[]

	return rs


    ######################################################################
    #  __postgresExecute
    ######################################################################
    def __postgresExecute(self,query,connection):

	# Execute Query
	try:
	    cursor=connection.query(query)
	except StandardError, e:
	    raise DBAccessException, '%s: %s' % (e,query)

	# Get Results
	if string.upper(query[0])=='S':	     # check for select
	    rs=cursor.getresult()
	    if not rs:
	        rs=[]
	else:
	    rs=[]

	return rs


    ######################################################################
    #  execute
    #
    #  execute query
    #      query -if query starts with 's' (select) then  return results 
    #             as a list of tuples, otherwise return empty list
    #      connection
    #            -if connection passed in then use it, otherwise
    #             open own connection
    ######################################################################
    def execute(self,query,connection=None):
	"""execute and return results of given query"""

	if self.debug:
	    print "============================="
	    print query
	    print "=============================" 

	# Obtain Connection
	if not connection is None:
	    conn = connection
	else:
	    conn = self.getConnection()

	# Execute query
	if self.dbtype==DBAccess.POSTGRES:
	    rs=self.__postgresExecute(query,conn)
	else:
	    rs=self.__oracleExecute(query,conn)

	# Close Connection
	if connection is None:
	    conn.close()

	return rs


    ######################################################################
    #  binExecute
    #
    #  execute query
    #      query -if query starts with 's' (select) then  return results 
    #             as a list of tuples, otherwise return empty list
    #      connection
    #            -if connection passed in then use it, otherwise
    #             open own connection
    #      x
    #            -binary data parameter
    #
    ######################################################################
    def binExecute(self,query,connection=None,x=''):
	"""execute and return results of given query, requires binary
           data in x parameter
	"""

	if self.debug:
	    print "============================="
	    print query
	    print "=============================" 

	# Obtain Connection
	if not connection is None:
	    conn = connection
	else:
	    conn = self.getConnection()

	if self.dbtype==DBAccess.POSTGRES:
	    pass
	    #rs=self.__postgresBinExecute(query,conn,x)
	else:
 	    rs=self.__oracleBinExecute(query,conn,x)

	# Close Connection
	if connection is None:
	    conn.close()

	return rs


    ######################################################################
    # getConnection()
    #
    # Returns a connection.
    ######################################################################
    def getConnection(self):
	'''return database connection'''
	if self.dbtype==DBAccess.POSTGRES:
	    return pg.connect(dbname=self.__dbname,host=self.__host,
		user=self.__user,passwd=self.__password,port=self.__port)
        return DCOracle2.Connect(self.__connectString)


    ######################################################################
    # closeConnection()
    #
    # Close the given connection
    ######################################################################
    def closeConnection(self,connection):
	'''close the given connection'''
	connection.close()


#END class DBAccess


