##############################################################################
# PTester.py - Persistence class test module.
#    classes:
#       PTester:  Parent class for unit test class for persistent objects
#                 generated by pcg
#
#  (c) 7x Group, LLC.
#            author:  Raymond Billings
#            created: 10/17/2001
#
##############################################################################
from whrandom import randint, random
from DBMap import FieldMap
#import unittest


###########################################################################
#  Tester.py
#
#  Classes and methods for performing tests
###########################################################################
#class PTester(unittest.TestCase):
class PTester:

    TWO_OR_FOUR=0
    TWO_DIGIT=1
    FOUR_DIGIT=2

    #######################################################
    #  constructor
    #######################################################
    def __init__(self,obj=None):
        self.obj=obj
	self.random=0

	# test options and defaults
	self.yearOption=PTester.FOUR_DIGIT


    #######################################################
    #  setObj
    #######################################################
    def setObj(self,obj=None):
        self.obj=obj

    
    #######################################################
    #  getObj
    #######################################################
    def getObj(self):
        return self.obj

    def setYearOption(self,value):
	self.yearOption=value
	
    def getYearOption(self):
	return self.yearOption

    def setRandom(self,value):
	self.random=value


    #######################################################
    #  runTests
    #######################################################
    def runTests(self):
	'''
	     run all tests against class
	'''
	if not self.obj:
	    print "WARNING(%s.runTests): No object reference, unable to test" \
		% (self.__class__.__name__)
	else:
	    self.testSetsAndGets()
	    self.testLists()
	    self.testDirectReferences()
	    self.testPersistence()
	    self.customTests()


    #######################################################
    #  testSetsAndGets()
    #
    #  abstract: needs to be overridden in subclasses
    #######################################################
    def testSetsAndGets(self):
	pass


    #######################################################
    #  testLists()
    #
    #  abstract: needs to be overridden in subclasses
    #######################################################
    def testLists(self):
	pass


    #######################################################
    #  testDirectReferences()
    #
    #  abstract: needs to be overridden in subclasses
    #######################################################
    def testDirectReferences(self):
	pass


    #######################################################
    #  customTests()
    #
    #  abstract: needs to be overridden in subclasses
    #######################################################
    def customTests(self):
	pass


    #######################################################
    #  testPersistence()
    #
    #######################################################
    def testPersistence(self):
	self.obj.setObjectId(0)
	self.obj.insert()
	print "\n%s.insert()" % (self.obj.__class__.__name__)
	self.obj.dump()
	self.set(testId=0)
	self.obj.restore()
	print "\n%s.restore()" % (self.obj.__class__.__name__)
	self.obj.dump()
	self.set(testId=0)
	self.obj.update()
	print "\n%s.update()" % (self.obj.__class__.__name__)
	self.obj.dump()
	self.obj.delete()
	self.obj.restore()
	print "\n%s.delete()" % (self.obj.__class__.__name__)
	self.obj.dump()


    #######################################################
    #  randomString
    #######################################################
    def randomString(self,length=20):
        symbols=['~', '`', '%', '#', '$', "'", '"', '-', '+', '=', '?']

        s=''
        l=randint(0,length)
        for i in range(l):
	    chartype=randint(1,100)
	    if chartype < 35:
	        letter=chr(randint(0,26) + ord('a'))
	    elif chartype < 70:
	        letter=chr(randint(0,26) + ord('A'))
	    elif chartype < 80:
	        letter=chr(randint(0,10) + ord('0'))
	    elif chartype < 90:
	        letter=' '
	    else:
	        letter=symbols[randint(0,len(symbols)-1)]

	    s=s+letter
	return s


    #######################################################
    #  getRandomValue
    #
    #  
    #######################################################
    def getValue(self,datatype=0,withQuote=0,size=20):
	'''Return random value depending on the type provided'''
	if self.random:
	    if datatype==FieldMap.INTEGER:
	        return self.randomInteger()
	    if datatype==FieldMap.FLOAT:
	        return self.randomFloat()
	    if datatype==FieldMap.DATE:
	        if withQuote:
		    return "'%s'" % self.validRandomDate()
	        else:
		    return self.validRandomDate()
	    if withQuote:
	        return "'%s'" % self.randomString(length=size)
	    return self.randomString(length=size)
	else:
	    if datatype==FieldMap.INTEGER:
	        return 100
	    if datatype==FieldMap.FLOAT:
	        return 3.14
	    if datatype==FieldMap.DATE:
	        if withQuote:
		    return "'2/29/2000'" 
	        else:
		    return '2/29/2000'
	    if withQuote:
	        return "'TEST_VALUE'" 
	    return 'TEST_VALUE'


    #######################################################
    #  invalidRandomDate
    #######################################################
    def invalidRandomDate(self):
	year=randint(-3000,3000)
	month=randint(-100,100)
	day=randint(-100,100)

	if randint(0,5)==0:
	    sYear=randomString(4)
	else:
	    sYear=str(year)
	if randint(0,5)==0:
	    sMonth=randomString(4)
	else:
	    sMonth=str(month)
	if randint(0,5)==0:
	    sDay=self.randomString(4)
	    sDay=str(day)

	if randint(0,100)<90:
	    delimiter='/'
	else:
	    delimiter='-'

	s=sDay
	if randint(0,100)<98:
	    s=s+delimiter
	s=s+sMonth
	if randint(0,100)<98:
	    s=s+delimiter
	s=s+sYear

	return s


    #######################################################
    #  validRandomDate
    #######################################################
    def validRandomDate(self,startYear=1900,endYear=2100,allowTwoDigit=0):
	monthDay=[31,28,31,30,31,30,31,31,30,31,30,31]

	year=randint(startYear,endYear)
	if year % 400 == 0 or  \
	   year % 4 == 0 and year % 100 != 0:
	    monthDay[1]=29
	month=randint(0,11)
	day=randint(0,monthDay[month])+1
	month=month+1

	if allowTwoDigit and randint(0,3)==0:
	    year=year%100

	return str(month) + "/" + str(day) + "/" + self.__yearStr(year)


    #######################################################
    #  randomDate
    #######################################################
    def randomDate(self,allowInvalid=0,startYear=1900,endYear=2100):
	if allowInvalid and randInt(0,3)==0:
	    return self.invalidRandomDate()
	return self.validRandomDate(startYear=startYear,endYear=endYear)


    #######################################################
    #  __yearStr
    #######################################################
    def __yearStr(self,year):
        s=''
	if self.yearOption==PTester.FOUR_DIGIT:
	    if year < 50:
	        s=str(year+2000)
	    elif year < 200:
		s=str(year+1900)
	    else:
	        s=str(year)
	elif self.yearOption==PTester.TWO_DIGIT:
	    s=str(year % 100)
	else:
	    s=str(year)

	return s


    #######################################################
    #  randomFloat
    #######################################################
    def randomFloat(self,start=-1000,end=1000):
	'''
	   return random floating point value

	   parameters:
		start: beginning of range (default - 1000)
		end:   end of range (default 1000)
	'''
	f=random()*(end-start)+start
	return f


    #######################################################
    #  set
    #######################################################
    def set(self,testId=1):
	idProperty=self.obj.getOidName()
	properties=self.obj.getPropertiesAsDictionary()
	for p in properties.keys():
	    datatype=self.obj.getType(p)
	    value=self.getValue(datatype)
	    if testId or p!=idProperty:
	        setattr(self.obj,p,value)


    #######################################################
    #  randomInt
    #######################################################
    def randomInteger(self,start=-1000,end=1000):
	'''
	   return random integer value

	   parameters:
		start: beginning of range (default -1000)
		end:   end of range (default 1000)
	'''
	i=randint(start,end)
	return i
	

    #######################################################
    #  check
    #######################################################
    def check(self,msg,condition):
	if not condition:
	    print "CHECK FAILED - " + msg

#End Class PTester


