##############################################################################
# Persist.py - Persistence module.
#
# NOTE:  Persist.py is now obsolete.. deprecated... decremented...
#        depreciated... and decimated...
#
#        It has been replaced by PersistentObject.py, DBMap.py, DBAccess.py
#        and Query.py
#
#    classes:
#	FieldMap:           database field/ class member variable mapping
#       DBMap:		    database table, id value, class, and fieldmap
#       PersistenceObject:  database functionality, parent class for 
#                           all persistence objects
#
#  (c) 7x Group, LLC.
#            author:  Chris Little, Larry Tuck, and Raymond Billings
#            created: 7/19/2001
#
##############################################################################
import types		# required for type comparisons
try:
    import DCOracle2        # required for connection to Oracle
except StandardError:
    pass
import string		# for upper function
from ObjectFactory import ObjectFactory


##############################################################################
# FieldMap
#
# Responsibilies:
#    - Map object attributes to table fields for a single class.
#
# Maps attributes to database fields
#
# dictionary:  Key            = property name
#              Value tuple[0] = column name
#                    tuple[1] = type: int, string, date, float
##############################################################################
class FieldMap:
    '''Implements support for tying object properties to a DB'''
    
    def __init__(self, fieldMap = {}):
        self.fieldMap = fieldMap

    def listColumnsAsString(self):
        '''Get a comma seperated list of fields'''

        s = ""
        separator = ""
        for property in self.fieldMap.keys():
            s = s + separator + self.fieldMap[property][0]
            separator = ","

        return s

    def listColumns(self):
	"get database columns as a list"
	fieldList=[]
	for i in self.fieldMap.values():
	    fieldList.append(i[0]);
	return fieldList

    def listProperties(self):
	"get properties as a list"
	return self.fieldMap.keys()

    def getType(self,property):
	"return type, given property"
	return self.fieldMap[property][1]

    def getInitValue(self,property):
	"return initialization value, given property"
	tp=self.fieldMap[property][1]
	if tp=='int':
	    return 0
	elif tp=='float':
	    return 0.0
	else:
	    return ''

    def getColumn(self,property):
	"return column name, given property"
	if self.fieldMap.has_key(property):
	    return self.fieldMap[property][0]
	else:
	    return ''

    def getValue(self,key):
	"return the tuple at the given key"

	return self.fieldMap[key]

    def union(self,fm):
        "union the given fieldmap to the current fieldmap"

	keys=fm.listProperties()
	for i in keys:
	    self.fieldMap[i]=fm.getValue(i)

    def dump(self):
	print self.fieldMap


# END class FieldMap
        

##############################################################################
#  DBMap
#
#  Responsibilities:
#     Maintain the mapping of object attributes to DB tables and fields for
#     an object and it's super classes.
#
#  Properties:
#
#    idLookup:  lookup string for the primary key
#    oidName: Name of object id, 'id' by default
#    keyName: Name of table primary key, 'ID' by default
#
#    dbMap:
#
#      List of tuples containing database information about class and class
#      hierarchy
#
#	tuple[0] = classname
#       tuple[1] = tablename
#       tuple[2] = Key Insert value string
#       tuple[3] = fieldmap for given table
#
#  Methods:
#     constructor:            initializes property
#     append:                 allow new table information to be appended
#     listTables:             return list of tables
#     getIdValueByTablename:  return id value for insert
#     getFieldMapByTablename: return appropriate field map
#     buildViewName:          build bottom level view name
#     buildViewFieldMap:      build entire fieldmap
#   
##############################################################################
class DBMap:
    "Map class information to database"

    def __init__(self):
        """constructor:  initialize map"""
	self.dbMap = []
	self.idLookup = ""
	self.oidName='id'
	self.keyName='ID'


    def append(self,clsOrTuple,tablename="",idValue="",fieldMap=None):
	"""append given tuple to map"""

	if type(clsOrTuple) == types.TupleType:
	    self.dbMap.append(clsOrTuple)

	else:
	    self.dbMap.append((clsOrTuple,tablename,idValue,fieldMap))


    def listTables(self):
	"""return list of tables within the db map"""

	rv = []
	for tuple in self.dbMap:
	    rv.append(tuple[1])

	return rv
	

    def getIdValueByTablename(self,tablename):
	"""return the insert id value given table name"""

	for tuple in self.dbMap:
	    if (tuple[1]==tablename):
		return tuple[2]

	return ""


    def getKeyName(self):
	"""return the primary key column name"""
	return self.keyName


    def setKeyName(self,name):
	"""set the primary key column name"""
	self.keyName=name


    def getOidName(self):
	"""return the object id name"""
	return self.oidName


    def setOidName(self,name):
	"""set the object id property name"""
	self.oidName=name


    def getIdLookup(self):
	"""return the lookup value for the primary key"""
	return self.idLookup

    def setIdLookupValue(self,idLookup):
	"""set the idLookup value"""
	self.idLookup=idLookup

    def getColumnName(self,property):
	"""return column name given property, '' if doesn't exist"""
	for tuple in self.dbMap:
	    fieldmap=tuple[3]
	    column=fieldmap.getColumn(property)
	    if column:
		return column
	return ''

    def getFieldMapByTablename(self,tablename):
	"""return the field map by the given tablename"""

	for tuple in self.dbMap:
	    if (tuple[1]==tablename):
		return tuple[3]

	return {}

    def buildViewName(self):
	"""construct a view name: it is the last table in list preceeded by v_"""
	
	lastTable=""
	for tuple in self.dbMap:
	    lastTable=tuple[1]

	return "v_"+lastTable

    def buildViewFieldMap(self):
	"""construct view fieldmap: all attributes of all tables"""
	
	fieldMap=FieldMap({})
	for tuple in self.dbMap:
	    fm=tuple[3]
	    fieldMap.union(fm)

	return fieldMap

    def dump(self):
	"""output map to screen"""
	print "idLookup: " + self.getIdLookup()
	print "DBMap:"
	for tuple in self.dbMap:
	    print 'CLASSNAME: ' + tuple[0]
	    print 'TABLENAME: ' + tuple[1]
            print 'KEY VALUE: ' + tuple[2]
	    tuple[3].dump()
	print ""

#END Class DBMap


##############################################################################
#  PersistentObject - The base class for classes that implement persistence .
#
#  Resposibilities:
#      - Abstract root class for all peristent objects.
#        This class DEFINES the interface for ALL persistent objects.
#        This class implements methods to insert, restore, update, and delete
#            objects from the persistent store.
#      
#  Interface: IPersistent
#    Doc: "Defines the interface for a persistent object."
#
#    Methods:
#        Name: restoreFromResult
#        ReturnType: NA
#        Doc: "Sets the persistent objects properties from a result (tuple or resultSet)."
#
#        Name: restore
#        ReturnType: NA
#        Doc: "Restores the object from the database."
#
#        Name: insert
#        ReturnType: NA
#        Doc: "Inserts the object in the database."
#
#        Name: update
#        ReturnType: NA
#        Doc: "Updates the object in the database."
#
#
##############################################################################
class PersistentObject:
    '''Base class for implementing object persistence.'''

    # restore, insert, update, delete queries
    sRestoreQueryTemplate = "SELECT %s FROM %s WHERE %s"
    sInsertQueryTemplate = "INSERT INTO %s (%s) VALUES (%s)"
    sUpdateQueryTemplate = "UPDATE %s SET %s WHERE %s"
    sDeleteQueryTemplate = "DELETE FROM %s WHERE %s"

    # list queries
    sListQueryTemplate = "SELECT %s FROM %s %s"
    sListByRowQueryTemplate = "SELECT %s FROM (select %s from %s where rownum <= %d %s) MINUS (select %s from %s where rownum < %d %s)"


    ######################################################################
    # Constructor
    ######################################################################
    def __init__(self,appName=""):
        self.dbMap = DBMap()

	self.__moduleName=''
	self.__connectString=''
	self.__applicationName=appName
	self.helper=None

	self.__setConnectString()
	self.__setModuleName()
        

    ######################################################################
    # setApplicationName
    #
    # set the name of the application - connect string will also be
    # reset
    ######################################################################
    def setApplicationName(self,appName):
	"""set the application name"""
	self.__applicationName=appName
	self.__setConnectString()
	self.__setModuleName()


    ######################################################################
    # getApplicationName
    #
    ######################################################################
    def getApplicationName(self):
	"""return the name of the application"""
	return self.__applicationName


    ######################################################################
    # __setConnectString
    #
    # currently a hardcoded if is used to determine the connect string
    # later this should be changed to a look up from a file with
    # very limited access
    ######################################################################
    def __setConnectString(self):
	if self.__applicationName=="Store":
	    self.__connectString="store/store@arctic"
	elif self.__applicationName=="Koniag":
	    self.__connectString="koniag/koniag@arctic"
	else:
	    self.__connectString="projadmin/projadmin@arctic"
	

    ######################################################################
    # __setModuleName
    #
    # currently hardcoded later will be lookup from file
    ######################################################################
    def __setModuleName(self):
	if self.__applicationName:
	    self.__moduleName=self.__applicationName


    ######################################################################
    # setHelper
    #
    # Set the helper attribute, which references a Helper object
    ######################################################################
    def setHelper(self,helper):
	"""set the helper attribute with the given helper object"""
	self.helper=helper

    ######################################################################
    # getHelper
    #
    # return the associated helper object
    ######################################################################
    def getHelper(self):
	"""return the associated helper object"""
	return self.helper


    ######################################################################
    # getMyType
    #
    # return classname
    ######################################################################
    def getMyType(self):
	"""return classname"""
	return self.__class__.__name__


    ######################################################################
    #  PersistentObject::restoreFromTuple(self,tuple,fieldMap)
    #
    #  REQUIRES:
    #      The order of the values in the tuple MUST match the order of the
    #      values in the properties in the fieldMap.  (The DCOracle connection
    #      does not provide a method for getting the value by column name.)
    #
    #  Given a field map and a tuple, restore properties in map
    ######################################################################
    def restoreFromTuple(self, tuple, fieldMap=None):
	'''Restore object from tuple and field map'''

        # if fieldMap is not specified, it means we are restoring from the
        # view.  Build the views fieldmap.
	if (fieldMap == None):
	    fieldMap = self.dbMap.buildViewFieldMap()

        properties=fieldMap.listProperties()
        index=0
        for property in properties:
	    value=tuple[index]

            # Replace Oracle 'missing values'
	    #if (value==Missing.Value):
	    if value is None:
	    	value=""

            # Convert integer values.
	    if (fieldMap.getType(property)=="int"):
	        if (type(value)!=types.IntType and value!=""):
		    value=int(value)

            # Convert float values
	    elif (fieldMap.getType(property)=="float"):
	        if (type(value)!=types.FloatType and value!=""):
		    value=float(value)

            # Now set the property on ourself with the value.
            setattr(self,property,value)

            # Move to the next index in the tuple.
            index=index + 1

	
    ######################################################################
    #  PersistentObject::restoreFromResult(self,rs,fieldMap)
    #
    #  Given a field map and a result, restore properties in map
    #  rs can be either a single tuple, or a list of tuples or lists
    #
    #  if a list, only the first element will be restored
    #
    #  A field map would only get passed in from within this class
    #  and would be used to restore a subset of the attributes from
    #  the result set.  Look at the restore() method for further
    #  detail.
    ######################################################################
    def restoreFromResult(self, rs, fieldMap=None):
        '''Restore the object from tuple or top row of a resultSet.'''

        # if fieldMap is not specified, it means we are restoring from the
        # view.  Build the views fieldmap.
	if (fieldMap == None):
	    fieldMap = self.dbMap.buildViewFieldMap()

	# set properties off of first of list of dictionaries, tuples, or lists
	if (type(rs) == types.ListType and rs):
            self.restoreFromTuple(rs[0], fieldMap)

	# set properties given no result
	else:
            for property in fieldMap.listProperties():
	 	initValue=fieldMap.getInitValue(property)
		setattr(self,property,initValue)


    ######################################################################
    #  PersistentObject::restore
    #
    #  restore the object from the persistent store.
    ######################################################################
    def restore(self):
        '''Restore the object from the persistent store.'''

	#  Connection
	conn=self.getConnection()
	cursor=conn.cursor()

	tables=self.dbMap.listTables()
	for t in tables:
	    fm=self.dbMap.getFieldMapByTablename(t)
	    query=self.buildRestoreStatement(t,fm)
	    print query
	    cursor.execute(query)
	    rs=cursor.fetchall()
	    self.restoreFromResult(rs,fm)
	conn.close()
	

    ######################################################################
    #  PersistentObject::insert
    #
    #  Insert the object into the persistent store.  
    #  Set the id of this object
    ######################################################################
    def insert(self):
        '''Insert the object into the persistent store.'''

        #  Connection
	conn=self.getConnection()
	cursor=conn.cursor()

	tables=self.dbMap.listTables()
	for t in tables:
	    fm=self.dbMap.getFieldMapByTablename(t)
	    insertValue=self.dbMap.getIdValueByTablename(t)
	    query=self.buildInsertStatement(t,insertValue,fm)
	    print query
	    cursor.execute(query)

	# set the id of the object
	idLookup=self.dbMap.getIdLookup()
	query="select %s from %s where rownum = 1" % (idLookup,tables[0])
	print query
	cursor.execute(query)
	rs=cursor.fetchall()
	if rs:
	    oid=int(rs[0][0])
	    setattr(self,self.dbMap.getOidName(),oid)

	conn.commit()
	conn.close()
        

    ######################################################################
    #  PersistentObject::update
    #
    #  Update the object in the persistent store.
    ######################################################################
    def update(self):
        '''Update the object in the persistent store.'''

        #  Connection
	conn=self.getConnection()
	cursor = conn.cursor()

	tables=self.dbMap.listTables()
	for t in tables:
	    fm=self.dbMap.getFieldMapByTablename(t)
	    query=self.buildUpdateStatement(t,fm)
	    print query
	    cursor.execute(query)
	conn.commit()
	conn.close()


    ########################################################################
    #  PersistentObject::delete()
    #
    #  Delete the object from the persistent store.
    ########################################################################
    def delete(self):
        '''Delete the object from the from the persistent store.'''

        #  Connection
	conn=self.getConnection()
	cursor=conn.cursor()

	tables=self.dbMap.listTables()
	for t in tables:
	    query=self.buildDeleteStatement(t)
	    print query
	    cursor.execute(query)

	self.deleteAssociations(cursor)
	conn.commit()
	conn.close()


    ########################################################################
    #  deleteAssociations(conn)
    #
    #  Abstract method to delete associations.  The correct delete
    #  queries will have to be put together and executed in the
    #  deleteAssociations method written within the subclass
    ########################################################################
    def deleteAssociations(self,cursor):
	"""abstract method for deletion of associated rows"""
	return


    ########################################################################
    #  asObjects
    #
    #  Return list of objects from resultset
    ########################################################################
    def asObjects(self, rs):
        '''Return list of objects from a resultset.'''

	factory = ObjectFactory(self.__moduleName)

	appName=self.getApplicationName()
	if not appName:
	    print "WARNING: missing application name for " + self.getMyType()
        objectList=[]
	if rs:
          for row in rs:
            object = factory.instantiate(self.getMyType())
	    if hasattr(object,'setApplicationName'):
	        object.setApplicationName(appName)
	    else:
	        print "WARNING: application name not set on new " + self.getMyType()
            object.restoreFromTuple(row)
	    helper=object.getHelper()
	    if helper:
		object=helper.restore(object)
		object.setApplicationName(appName)
            objectList.append(object)

        return objectList


    ######################################################################
    #  PersistentObject::execute
    #
    #  execute query
    #      -if query starts with 's' (select) then  return results 
    #       as a list of tuples, otherwise return empty list
    #
    ######################################################################
    def execute(self,query):
	"""execute and return results of given query"""

	conn = self.getConnection()
	cursor=conn.cursor()
	print query
	cursor.execute(query)
	if query[0]=='s' or query[0]=='S':	# check for select
	    rs=cursor.fetchall()
	    if not rs:
	        rs=[]
	else:
	    rs=[]
	conn.close()

	return rs


    ######################################################################
    # getConnection()
    #
    # (I created this method to consolidate the connection code
    #  which is driver speciifc.)
    #
    # Returns a connection.
    ######################################################################
    def getConnection(self):
	'''return oracle connection'''

        return DCOracle2.Connect(self.__connectString)


    ########################################################################
    #  listObjects
    #
    #  return list of objects matching condition
    ########################################################################
    def listObjects(self,criteria,exact=0):
        '''Restore list of objects using specified criteria.'''

        rs = self.list(criteria,exact=exact)
        objectList = criteria.asObjects(rs)

        return objectList


    ######################################################################
    #  list
    #
    #  basic list
    #
    #  criteria an object that represents the search criteria that
    #  should be generated as well as the objects types that should
    #  be generated.  Currently the criteria object is a regular
    #  business object but should eventually be changed to be a 
    #  separate critera object that give more flexibility in the
    #  development of search criteria
    ######################################################################
    def list(self,criteria="",exact=0):
        '''Get list of tuples from database using specified criteria.'''

	if (type(criteria) == types.InstanceType):
	    query = criteria.buildListStatement(criteria,exact=exact)
	else:
	    query = self.buildListStatement(criteria)
	return self.execute(query)


    ######################################################################
    #  listByRow
    #
    #  list by explicit row numbers
    ######################################################################
    def listByRow(self,start,end,condition=""):
        '''Get list of tuples by row from database.'''

	query=self.buildListByRowStatement(start,end,condition)
        return self.execute(query)


    ######################################################################
    #  Build sql statement strings
    ######################################################################
    def buildRestoreStatement(self,tablename,fieldMap):
	fields=self.buildColumnListForSelect(fieldMap)
	keyName=self.dbMap.getKeyName()
	oidName=self.dbMap.getOidName()

	stmt = PersistentObject.sRestoreQueryTemplate % \
	(fields, tablename, keyName + " = " + `getattr(self,'%s' % oidName)`)
	return stmt

    def buildInsertStatement(self,tablename,insertValue,fieldMap):
	fields=self.buildColumnListForInsert(fieldMap)
	values=self.buildValueListForInsert(insertValue,fieldMap)
	
	stmt = PersistentObject.sInsertQueryTemplate % \
	(tablename, fields, values)
	return stmt

    def buildUpdateStatement(self,tablename,fieldMap):
	assignments=self.buildAssignmentListForUpdate(fieldMap)
	keyName=self.dbMap.getKeyName()
	oidName=self.dbMap.getOidName()

	stmt = PersistentObject.sUpdateQueryTemplate % \
	(tablename, assignments, keyName + " = " + `getattr(self,'%s' % oidName)`)
	return stmt

    def buildDeleteStatement(self,tablename):
	keyName=self.dbMap.getKeyName()
	oidName=self.dbMap.getOidName()

	stmt = PersistentObject.sDeleteQueryTemplate % \
	(tablename, keyName + " = " + `getattr(self,'%s' % oidName)`)
	return stmt

    ######################################################################
    #  Build sql statement strings for lists
    ######################################################################
    def buildListStatement(self,condition="",exact=0):
	"Builds sql statement to pull a list of objects"

	fields=self.buildColumnListForSelect()
        viewname=self.dbMap.buildViewName()

	if type(condition) == types.InstanceType:
	    condition=self.buildCondition(condition,exact=exact)

	if (condition==""):
	    stmt = PersistentObject.sListQueryTemplate % \
	    (fields, viewname, "")
	else:
	    whereClaus = " where " + condition
	    stmt = PersistentObject.sListQueryTemplate % \
	    (fields,viewname,whereClaus)
	return stmt


    def buildListByRowStatement(self, start, end, condition="",exact=0):
	"Builds sql statement to pull list of objects by row number"

	fields=self.buildColumnListForSelect()
        viewname=self.dbMap.buildViewName()

	if (type(condition) == types.InstanceType):
	    condition=self.buildCondition(condition,exact=exact)

	if (condition==""):
	    stmt = PersistentObject.sListByRowQueryTemplate % \
	    (fields,fields,viewname,end,condition,fields,viewname,start,"")
	else:
	    condition=" and " + condition
	    stmt = PersistentObject.sListByRowQueryTemplate % \
	    (fields,fields,viewname,end,condition,fields,viewname,start,condition)

	return stmt
   

    ######################################################################
    #  buildAssignmentListForUpdate
    #
    #  build comma delimited string of assignments
    ######################################################################
    def buildAssignmentListForUpdate(self,fieldMap):
        '''Get the assignments for fields for update.'''

        s = ""
        separator = ""
        for property in fieldMap.listProperties():
	    dataType = fieldMap.getType(property)
	    column = fieldMap.getColumn(property)
	    if (dataType == "date"):
                s = s + separator + column + " = to_date('" + getattr(self, property) + "','MM/DD/YYYY')"
	    elif (dataType == "string"):
                s = s + separator + column + " = '" + \
		    self.__convertStringForUpdate(getattr(self, property)) + "'"
	    elif (dataType == "text"):
                s = s + separator + column + " = '" + \
		    self.__convertTextForUpdate(getattr(self, property)) + "'"
	    else:
                s = s + separator + column + '=' + `getattr(self, property)`
            separator = ","

        return s


    ######################################################################
    #  buildColumnListForSelect
    #
    #  build comma delimited string of columns with date conversion
    #
    #  If parameter is field map then use it, otherwise generate
    #  one from dbMap, if parameter is string then assume that it
    #  is a column prefix for use by join queries
    ######################################################################
    def buildColumnListForSelect(self,inParm=""):
        '''Get a comma seperated list of fields with date conversion'''

	if (type(inParm) == types.StringType):
	    colPrefix=inParm
	    fieldMap=self.dbMap.buildViewFieldMap()
	else:
	    colPrefix=""
	    fieldMap=inParm

        s = ""
        separator = ""
        for property in fieldMap.listProperties():
	    column = colPrefix + fieldMap.getColumn(property)
	    if (fieldMap.getType(property) == "date"):
                s = s + separator + "to_char(" + column + ",'MM/DD/YYYY')"
	    else:
                s = s + separator + column
            separator = ","

        return s


    ######################################################################
    #  buildColumnListForInsert
    #
    #  build comma delimited string of columns, no conversions
    ######################################################################
    def buildColumnListForInsert(self,fieldMap):
	return fieldMap.listColumnsAsString()


    ######################################################################
    #  buildValueListForInsert
    #
    #  build comma delimited string of values with date conversions
    ######################################################################
    def buildValueListForInsert(self,idValue,fieldMap):
        '''Get the values of fields for insert less id '''

        s = ""
        separator = ""
	oidName=self.dbMap.getOidName()
        for property in fieldMap.listProperties():
  	    if property==oidName and idValue!="":
                s = s + separator + idValue
	    else:
	        if (fieldMap.getType(property) =="date"):
                    s = s + separator + "to_date('" + getattr(self, property) + "','MM/DD/YYYY')"
	        elif fieldMap.getType(property)=="string":
                    s = s + separator + "'" + \
			self.__convertStringForUpdate(getattr(self, property)) + "'"
	        elif fieldMap.getType(property)=="text":
                    s = s + separator + "'" + \
			self.__convertTextForUpdate(getattr(self, property)) + "'"
	        else:
                    s = s + separator + `getattr(self, property)`
            separator = ","

        return s


    ######################################################################
    #  buildCondition 
    #
    #  build search condition based on object values
    ######################################################################
    def buildCondition(self,object,colPrefix="",exact=0):
	'''Return condition based on set values of object.'''

	fieldMap=self.dbMap.buildViewFieldMap()

	condition=""
        prefix=""
	for property in fieldMap.listProperties():
	    value=getattr(object,property)
	    pType=fieldMap.getType(property)
	    column=colPrefix+fieldMap.getColumn(property)
	    if (value):
	        if (pType=="date"):
		    condition=condition + prefix + column + " = to_date('" + value + "','MM/DD/YYYY')"
		    prefix = " and " 
		elif (pType=="int" or pType=="float"):
		    condition=condition + prefix + column + " = " + `value`
		    prefix = " and " 
		elif exact:
		    condition=condition + prefix + column + " = '" + value + "'"
		else:
		    condition=condition + prefix + "upper(" + column + ") like '" \
			+ string.upper(value) + "%'"
	        prefix=" and "

	return condition


    ######################################################################
    #  __convertTextForUpdate
    #
    #  The following conversions will be made:
    #      - '\n' will be converted to new line
    #      - single quote will be converted to 3 single quotes
    ######################################################################
    def __convertTextForUpdate(self,str):
	s1 = string.replace(str,'\\n','\n')
	s2 = self.__convertStringForUpdate(self,s1)
	return s2


    ######################################################################
    #  __convertStringForUpdate
    #
    #  The following conversions will be made:
    #      - single quote will be converted to 3 single quotes
    ######################################################################
    def __convertStringForUpdate(self,str):
	s1 = string.replace(str,"'","''")
	return s1


    ######################################################################
    #  getViewName
    #
    #  return the view name , delegates to dbMap
    ######################################################################
    def getViewName(self):
	"""return view name for this object"""
	return self.dbMap.buildViewName()


    ######################################################################
    #  getColumnName
    #
    #  return the column name for the given property name
    ######################################################################
    def getColumnName(self,propertyName):
	"""return column name for the given propertyName"""
	return self.dbMap.getColumnName(propertyName)


#END class PersistentObject

